
#!/usr/local/bin/perl
#
# Copyright (C) 2003,2004 Alexandre Dulaunoy <adulau@foo.be>
#
# This program is  free software; you can redistribute  it and/or modify
# it under the  terms of the GNU General Public  License as published by
# the Free Software Foundation; either  version 2 of the License, or (at
# your option) any later version.
#
# This program  is distributed in the  hope that it will  be useful, but
# WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# General Public License for more details.
#
# You  should have received  a copy  of the  GNU General  Public License
# along  with  this  program;  if   not,  write  to  the  Free  Software
# Foundation, Inc., 59 Temple Place  - Suite 330, Boston, MA 02111-1307,
# USA.
#
# This software is a sample application to demonstrate the use of MQS.
#
# $Id: getpolicy.pl,v 1.1 2004/12/29 12:27:15 adulau Exp adulau $

use RPC::XML::Client;
use Data::Dumper;
use Getopt::Long;
use Digest::SHA1 qw(sha1_hex);
$| = 1;

my $version   = "0.0.1";
my $copyright =
"Copyright (C) 2003,2004 Alexandre Dulaunoy <adulau\@foo.be> and released under the GNU General Public License.";

my $result = GetOptions(
    "mqsurl=s" => \$mqsurl,
    "auth=s"   => \$auth,
    "output=s" => \$outputfile,
    "target=s" => \$target
);

if ( !( defined($outputfile) ) ) { my $outputfile = "pf.conf"; }

if (   !( defined($mqsurl) )
    || !( defined($auth) )
    || !( defined($target) ) )
{
    print "getpolicy.pl $version usage :\n";
    print "\t--mqsurl <mqsurl> (e.g. https://127.0.0.1:9001/)\n";
    print
      "\t--auth <mqsauthstring> (e.g. auth:simple:test:anotherbadsamplekey)\n";
    print
"\t--target <targetofthepolicy> (e.g. fw1-external) This is the queue name dedicated to the target.\n";
    print
"\t--output <outputfile> (e.g. pf.conf) If the fetch from the server is successful then\n";
    print "\twrite a file containing the policy.";
    print "\n$copyright\n";
    exit(-1);
}

my $client = new RPC::XML::Client $mqsurl;

chomp($policystring);

#print "Get policy $policystring --";
my $req   = RPC::XML::request->new( 'mqs.GetMessage', $auth, $target );
my $res   = $client->send_request($req);
my $value = $res->value;

if ( ref($value) eq "HASH" ) {
    print Dumper( \$value );
    die "XML-RPC error";
}

open( FILEX, "<", \$value );

my $i;
my $entity;
my $policy;

while (<FILEX>) {
    if ( $i == 0 ) {
        $entity = $_;
    }
    else {
        if ( $_ =~ /^\+\+\+/ ) {
            last;
        }
        else {
            $policy .= $_;
        }
    }

    $i++;
}

my @entityvalue = split( /\;/, $entity );
chomp($policy);

chomp( $entityvalue[2] );

my $filedigest = sha1_hex($policy);

if ( $filedigest eq $entityvalue[2] ) {
    open( FILEY, ">$outputfile" ) or die "Unable to write output file";
    print FILEY $policy;
    close(FILEY);
}

close(FILEX);
